
data study;
input 
id    :   $3.
visit :   mmddyy.
chol 
;
format visit mmddyy10.;
datalines;
001    10/15/2004      200
002    10/15/2004      200
003    10/15/2004      300
004    10/15/2004      275
005    10/15/2004      250
002    11/10/2004      175
002    11/10/2004      175
002    11/10/2004      175
002    11/10/2004      195
004    11/13/2004      275
003    11/14/2004      280
004    12/14/2004      275
;
run;

proc sort data=study;
by id visit;
run;

proc print data=study;
run;

* add FIRST. and LAST. variables to the data set;
data fl;
set study;
by id;
put id visit first.id last.id;
*if last.id then put;
run;



* add FIRST. and LAST. variables to the data set;
data fl;
set study;
by id;
first_id = first.id;
last_id  = last.id;
label
first_id = 'first.id'
last_id  = 'last.id'
;
run;

proc print data=fl label;
run;


* first time you saw each ID;
data study_f;
set study;
by id;
if first.id;
run;

title 'FIRST VISIT';
proc print data=study_f;
run;

* last time you saw each ID;
data study_l;
set study;
by id;
if last.id;
run;

title 'LAST VISIT';
proc print data=study_l;
run;

* first and last time you saw each ID;
data study_fl;
set study;
by id;
if first.id or last.id;
run;

title 'FIRST AND LAST VISIT';
proc print data=study_fl;
run;

* separate data sets for single and multiple visit subjects;
data single multiple;
set study;
by id;
if first.id and last.id then output single;
else output multiple;
run;

title 'SINGLE VISIT';
proc print data=single;
run;

title 'MULTIPLE VISITS';
proc print data=multiple;
run;

* first time you saw each ID only for those with multiple visits;
data study_fm;
set study;
by id;
if first.id and not last.id;
run;

title 'FIRST VISIT OF MULTIPLE VISITS';
proc print data=study_fm;
run;

* add FIRST. and LAST. variables to the data set;
data fl;
set study;
by id visit;
first_id = first.id;
last_id  = last.id;
first_visit = first.visit;
last_visit  = last.visit;
label
first_id = 'first.id'
last_id  = 'last.id'
first_visit = 'first.visit'
last_visit  = 'last.visit'
;
run;

proc print data=fl label;
run;

* repeated dates within an ID;
data repeats;
set study;
by id visit;
if not (first.visit and last.visit);
run;

title 'REPEATED VISITS WITHIN AN ID';
proc print data=repeats;
run;

* another approach --- slightly different problem;
proc sort data=study out=repeats_no dupout=repeats_yes nodupkey;
by id visit;
run;

title 'UNIQUE VISITS WITHIN AN ID';
proc print data=repeats_no;
run;

title 'REPEATED VISITS WITHIN AN ID';
proc print data=repeats_yes;
run;

* another question ... compute difference between first CHOL and last CHOL;
data study;
input 
id    :   $3.
visit :   mmddyy.
chol 
;
format visit mmddyy10.;
datalines;
001    10/15/2004      200
002    10/15/2004      200
003    10/15/2004      300
004    10/15/2004      275
005    10/15/2004      250
002    11/10/2004      175
002    11/10/2004      175
002    11/10/2004      175
002    11/10/2004      195
004    11/13/2004      275
003    11/14/2004      280
004    12/14/2004      275
;
run;

proc sort data=study;
by id visit;
run;

data diff;
retain firstchol;  
set study;
by id;  
if first.id then firstchol = chol;  
if last.id then do;  
   diffchol = chol - firstchol;  
   output;  
end;
run;

title 'DATA SET DIFF:  DIFFERENCE IN CHOLESTEROL';
proc print data=diff;
var id firstchol chol diffchol visit;
run;

* add a count of the number of visits;
data diff;
retain firstchol;
set study;
by id;
if first.id then do;  
   firstchol = chol;
   nvisits = 0;  
end;
nvisits + 1;  
if last.id then do;
   diffchol = chol - firstchol;
   output;
end;
run;

title 'DATA SET DIFF:  DIFFERENCE IN CHOLESTEROL';
proc print data=diff;
var id nvisits firstchol chol diffchol visit;
run;

* keep track of how many days a person was in the study;

data diff;
retain firstchol firstvisit;  
set study;
by id;
if first.id then do; 
   firstchol = chol;
   firstvisit = visit;  
   nvisits = 0;
end;
nvisits + 1;
if last.id then do;
   diffchol = chol - firstchol;
   diffdays = visit - firstvisit;  
   output;
end;
format firstvisit mmddyy10.;
run;

title 'DATA SET DIFF:  DIFFERENCE IN CHOLESTEROL, NUMBER OF VISITS, AND DAYS IN THE STUDY';
proc print data=diff;
var id nvisits firstchol chol diffchol firstvisit visit diffdays;
run;

* use a date constant since the first visit is the same for each subject;

data diff;
retain firstchol;
set study;
by id;
if first.id then do; 
   firstchol = chol;
   nvisits = 0;
end;
nvisits + 1;
if last.id then do;
   diffchol = chol - firstchol;
   diffdays = visit - '15oct2004'd;  
   output;
end;
run;

title 'DATA SET DIFF:  DIFFERENCE IN CHOLESTEROL, NUMBER OF VISITS, AND DAYS IN THE STUDY';
proc print data=diff;
var id nvisits firstchol chol diffchol visit diffdays;
run;

* 'sophisticated';

data diff;
do until (last.id);
  set study;
  by id;
  if first.id then firstchol = chol;
  nvisits = sum(nvisits,1);
end;
diffchol = chol - firstchol;
diffdays = visit - '15oct2004'd;  
run;

title 'DATA SET DIFF:  DIFFERENCE IN CHOLESTEROL, NUMBER OF VISITS, AND DAYS IN THE STUDY';
proc print data=diff;
var id nvisits firstchol chol diffchol visit diffdays;
run;





* different data - should ONLY be one observation per social security number;

* make some fake data;
data ssns;
do j=1 to 1e5;
   rnd = ranuni(9999);
   ssn = ceil(300000000*rnd);
   output;
end;
keep ssn;
format ssn ssn.;
run;

proc print data=ssns (obs=10);
run;

proc sort data=ssns;
by ssn;
run;

data ssn_many;
set ssns;
by ssn;
if not (first.ssn and last.ssn);
run;

title 'REPEAT SOCIAL SECURITY NUMBERS';
proc print data=ssn_many;
run;

proc freq data=ssn_many;
table ssn / noprint out=temp;
run;

proc print data=temp;
run;

proc freq data=temp;
table count;
run;
